@@ -1,13 +1,14 @@
 use crate::auditwheel::PlatformTag;
 use crate::build_context::BridgeModel;
+use crate::compile::{CompileTarget, LIB_CRATE_TYPES};
 use crate::cross_compile::{find_sysconfigdata, parse_sysconfigdata};
 use crate::project_layout::ProjectResolver;
 use crate::pyproject_toml::ToolMaturin;
 use crate::python_interpreter::{InterpreterConfig, InterpreterKind, MINIMUM_PYTHON_MINOR};
-use crate::{BuildContext, Metadata21, PythonInterpreter, Target};
+use crate::{BuildContext, PythonInterpreter, Target};
 use anyhow::{bail, format_err, Context, Result};
 use cargo_metadata::{Metadata, Node};
-use regex::Regex;
+use pep440_rs::VersionSpecifiers;
 use serde::{Deserialize, Serialize};
 use std::collections::{HashMap, HashSet};
 use std::env;
@@ -153,8 +154,8 @@ pub struct BuildOptions {
     )]
     pub platform_tag: Vec<PlatformTag>,
 
-    /// The python versions to build wheels for, given as the names of the
-    /// interpreters.
+    /// The python versions to build wheels for, given as the executables of
+    /// interpreters such as `python3.9` or `/usr/bin/python3.8`.
     #[arg(short, long, num_args = 0.., action = clap::ArgAction::Append)]
     pub interpreter: Vec<PathBuf>,
 
@@ -180,6 +181,7 @@ pub struct BuildOptions {
     /// Default to manylinux2014/manylinux_2_17 if you do not specify an `--compatibility`
     ///
     /// Make sure you installed zig with `pip install maturin[zig]`
+    #[cfg(feature = "zig")]
     #[arg(long)]
     pub zig: bool,
 
@@ -214,7 +216,7 @@ impl BuildOptions {
         bridge: &BridgeModel,
         interpreter: &[PathBuf],
         target: &Target,
-        min_python_minor: Option<usize>,
+        requires_python: Option<&VersionSpecifiers>,
         generate_import_lib: bool,
     ) -> Result<Vec<PythonInterpreter>> {
         match bridge {
@@ -230,12 +232,8 @@ impl BuildOptions {
                     interpreters.push(PythonInterpreter::from_config(interpreter_config));
                 } else if binding_name.starts_with("pyo3") && target.cross_compiling() {
                     if let Some(cross_lib_dir) = env::var_os("PYO3_CROSS_LIB_DIR") {
-                        let host_interpreters = find_interpreter_in_host(
-                            bridge,
-                            interpreter,
-                            target,
-                            min_python_minor,
-                        )?;
+                        let host_interpreters =
+                            find_interpreter_in_host(bridge, interpreter, target, requires_python)?;
                         let host_python = &host_interpreters[0];
                         eprintln!("🐍 Using host {host_python} for cross-compiling preparation");
                         // pyo3
@@ -311,14 +309,14 @@ impl BuildOptions {
                             }
                         }
                         interpreters =
-                            find_interpreter_in_sysconfig(interpreter, target, min_python_minor)?;
+                            find_interpreter_in_sysconfig(interpreter, target, requires_python)?;
                     }
                 } else if binding_name.starts_with("pyo3") {
                     // Only pyo3/pyo3-ffi bindings supports bundled sysconfig interpreters
-                    interpreters = find_interpreter(bridge, interpreter, target, min_python_minor)?;
+                    interpreters = find_interpreter(bridge, interpreter, target, requires_python)?;
                 } else {
                     interpreters =
-                        find_interpreter_in_host(bridge, interpreter, target, min_python_minor)?;
+                        find_interpreter_in_host(bridge, interpreter, target, requires_python)?;
                 }
 
                 let interpreters_str = interpreters
@@ -342,13 +340,9 @@ impl BuildOptions {
                     // Ideally, we wouldn't want to use any python interpreter without abi3 at all.
                     // Unfortunately, on windows we need one to figure out base_prefix for a linker
                     // argument.
-                    let interpreters = find_interpreter_in_host(
-                        bridge,
-                        interpreter,
-                        target,
-                        Some(*minor as usize),
-                    )
-                    .unwrap_or_default();
+                    let interpreters =
+                        find_interpreter_in_host(bridge, interpreter, target, requires_python)
+                            .unwrap_or_default();
                     if env::var_os("PYO3_CROSS_LIB_DIR").is_some() {
                         // PYO3_CROSS_LIB_DIR should point to the `libs` directory inside base_prefix
                         // when cross compiling, so we fake a python interpreter matching it
@@ -400,26 +394,22 @@ impl BuildOptions {
                         bail!("Failed to find a python interpreter");
                     }
                 } else {
-                    let found_interpreters = find_interpreter_in_host(
-                        bridge,
-                        interpreter,
-                        target,
-                        Some(*minor as usize),
-                    )
-                    .or_else(|err| {
-                        let interps = find_interpreter_in_sysconfig(
-                            interpreter,
-                            target,
-                            Some(*minor as usize),
-                        )
-                        .unwrap_or_default();
-                        if interps.is_empty() && !self.interpreter.is_empty() {
-                            // Print error when user supplied `--interpreter` option
-                            Err(err)
-                        } else {
-                            Ok(interps)
-                        }
-                    })?;
+                    let found_interpreters =
+                        find_interpreter_in_host(bridge, interpreter, target, requires_python)
+                            .or_else(|err| {
+                                let interps = find_interpreter_in_sysconfig(
+                                    interpreter,
+                                    target,
+                                    requires_python,
+                                )
+                                .unwrap_or_default();
+                                if interps.is_empty() && !self.interpreter.is_empty() {
+                                    // Print error when user supplied `--interpreter` option
+                                    Err(err)
+                                } else {
+                                    Ok(interps)
+                                }
+                            })?;
                     eprintln!("🐍 Not using a specific python interpreter");
                     if self.interpreter.is_empty() {
                         // Fake one to make `BuildContext::build_wheels` happy for abi3 when no cpython/pypy found on host
@@ -459,7 +449,7 @@ impl BuildOptions {
                             interps.extend(find_interpreter_in_sysconfig(
                                 &pypys,
                                 target,
-                                min_python_minor,
+                                requires_python,
                             )?)
                         }
                         if interps.is_empty() {
@@ -566,7 +556,7 @@ impl BuildOptions {
                 &bridge,
                 &[],
                 &target,
-                get_min_python_minor(&metadata21),
+                metadata21.requires_python.as_ref(),
                 generate_import_lib,
             )?
         } else {
@@ -599,6 +589,10 @@ impl BuildOptions {
         let skip_auditwheel =
             pyproject.map(|x| x.skip_auditwheel()).unwrap_or_default() || self.skip_auditwheel;
         let platform_tags = if self.platform_tag.is_empty() {
+            #[cfg(feature = "zig")]
+            let use_zig = self.zig;
+            #[cfg(not(feature = "zig"))]
+            let use_zig = false;
             let compatibility = pyproject
                 .and_then(|x| {
                     if x.compatibility().is_some() {
@@ -606,7 +600,7 @@ impl BuildOptions {
                     }
                     x.compatibility()
                 })
-                .or(if self.zig {
+                .or(if use_zig {
                     if target.is_musl_target() {
                         // Zig bundles musl 1.2
                         Some(PlatformTag::Musllinux { x: 1, y: 2 })
@@ -657,11 +651,15 @@ impl BuildOptions {
             .target_dir
             .clone()
             .unwrap_or_else(|| cargo_metadata.target_directory.clone().into_std_path_buf());
-        let crate_name = cargo_toml.package.name;
 
+        let config_targets = pyproject.and_then(|x| x.targets());
+        let compile_targets =
+            filter_cargo_targets(&cargo_metadata, bridge, config_targets.as_deref())?;
+
+        let crate_name = cargo_toml.package.name;
         Ok(BuildContext {
             target,
-            bridge,
+            compile_targets,
             project_layout,
             pyproject_toml_path,
             pyproject_toml,
@@ -674,6 +672,7 @@ impl BuildOptions {
             release,
             strip,
             skip_auditwheel,
+            #[cfg(feature = "zig")]
             zig: self.zig,
             platform_tag: platform_tags,
             interpreter,
@@ -733,27 +732,88 @@ fn validate_bridge_type(
     Ok(())
 }
 
-/// Uses very simple PEP 440 subset parsing to determine the
-/// minimum supported python minor version for interpreter search
-fn get_min_python_minor(metadata21: &Metadata21) -> Option<usize> {
-    if let Some(requires_python) = &metadata21.requires_python {
-        let regex = Regex::new(r#">=3\.(\d+)(?:\.\d)?"#).unwrap();
-        if let Some(captures) = regex.captures(requires_python) {
-            let min_python_minor = captures[1]
-                .parse::<usize>()
-                .expect("Regex must only match usize");
-            Some(min_python_minor)
-        } else {
-            eprintln!(
-                "⚠️ Couldn't parse the value of requires-python, \
-                    not taking it into account when searching for python interpreter. \
-                    Note: Only `>=3.x.y` is currently supported."
-            );
-            None
+fn filter_cargo_targets(
+    cargo_metadata: &Metadata,
+    bridge: BridgeModel,
+    config_targets: Option<&[crate::pyproject_toml::CargoTarget]>,
+) -> Result<Vec<CompileTarget>> {
+    let root_pkg = cargo_metadata.root_package().unwrap();
+    let resolved_features = cargo_metadata
+        .resolve
+        .as_ref()
+        .and_then(|resolve| resolve.nodes.iter().find(|node| node.id == root_pkg.id))
+        .map(|node| node.features.clone())
+        .unwrap_or_default();
+    let mut targets: Vec<_> = root_pkg
+        .targets
+        .iter()
+        .filter(|target| match bridge {
+            BridgeModel::Bin(_) => {
+                let is_bin = target.kind.contains(&"bin".to_string());
+                if target.required_features.is_empty() {
+                    is_bin
+                } else {
+                    // Check all required features are enabled for this bin target
+                    is_bin
+                        && target
+                            .required_features
+                            .iter()
+                            .all(|f| resolved_features.contains(f))
+                }
+            }
+            _ => target.kind.contains(&"cdylib".to_string()),
+        })
+        .map(|target| CompileTarget {
+            target: target.clone(),
+            bridge_model: bridge.clone(),
+        })
+        .collect();
+    if targets.is_empty() && !bridge.is_bin() {
+        // No `crate-type = ["cdylib"]` in `Cargo.toml`
+        // Let's try compile one of the target with `--crate-type cdylib`
+        let lib_target = root_pkg.targets.iter().find(|target| {
+            target
+                .kind
+                .iter()
+                .any(|k| LIB_CRATE_TYPES.contains(&k.as_str()))
+        });
+        if let Some(target) = lib_target {
+            targets.push(CompileTarget {
+                target: target.clone(),
+                bridge_model: bridge,
+            });
         }
-    } else {
-        None
     }
+
+    // Filter targets by config_targets
+    if let Some(config_targets) = config_targets {
+        targets.retain(|CompileTarget { target, .. }| {
+            config_targets.iter().any(|config_target| {
+                let name_eq = config_target.name == target.name;
+                match &config_target.kind {
+                    Some(kind) => name_eq && target.kind.contains(kind),
+                    None => name_eq,
+                }
+            })
+        });
+        if targets.is_empty() {
+            bail!(
+                "No Cargo targets matched by `package.metadata.maturin.targets`, please check your `Cargo.toml`"
+            );
+        } else {
+            let target_names = targets
+                .iter()
+                .map(|CompileTarget { target, .. }| target.name.as_str())
+                .collect::<Vec<_>>();
+            eprintln!(
+                "🎯 Found {} Cargo targets in `Cargo.toml`: {}",
+                targets.len(),
+                target_names.join(", ")
+            );
+        }
+    }
+
+    Ok(targets)
 }
 
 /// pyo3 supports building abi3 wheels if the unstable-api feature is not selected
@@ -935,6 +995,7 @@ pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<Br
 
     if !(bridge.is_bindings("pyo3") || bridge.is_bindings("pyo3-ffi")) {
         eprintln!("🔗 Found {bridge} bindings");
+        return Ok(bridge);
     }
 
     for &lib in PYO3_BINDING_CRATES.iter() {
@@ -994,7 +1055,7 @@ fn find_interpreter(
     bridge: &BridgeModel,
     interpreter: &[PathBuf],
     target: &Target,
-    min_python_minor: Option<usize>,
+    requires_python: Option<&VersionSpecifiers>,
 ) -> Result<Vec<PythonInterpreter>> {
     let mut interpreters = Vec::new();
     if !interpreter.is_empty() {
@@ -1007,17 +1068,17 @@ fn find_interpreter(
         }
         if !missing.is_empty() {
             let sysconfig_interps =
-                find_interpreter_in_sysconfig(&missing, target, min_python_minor)?;
+                find_interpreter_in_sysconfig(&missing, target, requires_python)?;
             interpreters.extend(sysconfig_interps);
         }
     } else {
-        interpreters = PythonInterpreter::find_all(target, bridge, min_python_minor)
+        interpreters = PythonInterpreter::find_all(target, bridge, requires_python)
             .context("Finding python interpreters failed")?;
     };
 
     if interpreters.is_empty() {
-        if let Some(minor) = min_python_minor {
-            bail!("Couldn't find any python interpreters with version >= 3.{}. Please specify at least one with -i", minor);
+        if let Some(requires_python) = requires_python {
+            bail!("Couldn't find any python interpreters with version {}. Please specify at least one with -i", requires_python);
         } else {
             bail!("Couldn't find any python interpreters. Please specify at least one with -i");
         }
@@ -1030,19 +1091,19 @@ fn find_interpreter_in_host(
     bridge: &BridgeModel,
     interpreter: &[PathBuf],
     target: &Target,
-    min_python_minor: Option<usize>,
+    requires_python: Option<&VersionSpecifiers>,
 ) -> Result<Vec<PythonInterpreter>> {
     let interpreters = if !interpreter.is_empty() {
         PythonInterpreter::check_executables(interpreter, target, bridge)
             .context("The given list of python interpreters is invalid")?
     } else {
-        PythonInterpreter::find_all(target, bridge, min_python_minor)
+        PythonInterpreter::find_all(target, bridge, requires_python)
             .context("Finding python interpreters failed")?
     };
 
     if interpreters.is_empty() {
-        if let Some(minor) = min_python_minor {
-            bail!("Couldn't find any python interpreters with version >= 3.{}. Please specify at least one with -i", minor);
+        if let Some(requires_python) = requires_python {
+            bail!("Couldn't find any python interpreters with {}. Please specify at least one with -i", requires_python);
         } else {
             bail!("Couldn't find any python interpreters. Please specify at least one with -i");
         }
@@ -1054,10 +1115,10 @@ fn find_interpreter_in_host(
 fn find_interpreter_in_sysconfig(
     interpreter: &[PathBuf],
     target: &Target,
-    min_python_minor: Option<usize>,
+    requires_python: Option<&VersionSpecifiers>,
 ) -> Result<Vec<PythonInterpreter>> {
     if interpreter.is_empty() {
-        return Ok(PythonInterpreter::find_by_target(target, min_python_minor));
+        return Ok(PythonInterpreter::find_by_target(target, requires_python));
     }
     let mut interpreters = Vec::new();
     for interp in interpreter {
@@ -1154,8 +1215,6 @@ impl From<CargoOptions> for cargo_options::Rustc {
                     None => Vec::new(),
                 },
                 target_dir: cargo.target_dir,
-                manifest_path: cargo.manifest_path,
-                ignore_rust_version: cargo.ignore_rust_version,
                 verbose: cargo.verbose,
                 color: cargo.color,
                 frozen: cargo.frozen,
@@ -1166,6 +1225,8 @@ impl From<CargoOptions> for cargo_options::Rustc {
                 timings: cargo.timings,
                 ..Default::default()
             },
+            manifest_path: cargo.manifest_path,
+            ignore_rust_version: cargo.ignore_rust_version,
             future_incompat_report: cargo.future_incompat_report,
             args: cargo.args,
             ..Default::default()
@@ -1400,21 +1461,4 @@ mod test {
 
         assert_eq!(extract_cargo_metadata_args(&args).unwrap(), expected);
     }
-
-    #[test]
-    fn test_get_min_python_minor() {
-        use crate::CargoToml;
-
-        // Nothing specified
-        let manifest_path = "test-crates/pyo3-pure/Cargo.toml";
-        let cargo_toml = CargoToml::from_path(manifest_path).unwrap();
-        let cargo_metadata = MetadataCommand::new()
-            .manifest_path(manifest_path)
-            .exec()
-            .unwrap();
-        let metadata21 =
-            Metadata21::from_cargo_toml(&cargo_toml, "test-crates/pyo3-pure", &cargo_metadata)
-                .unwrap();
-        assert_eq!(get_min_python_minor(&metadata21), None);
-    }
 }
