@@ -1,9 +1,12 @@
 use crate::build_context::BridgeModel;
 use crate::target::RUST_1_64_0;
-use crate::{BuildContext, PlatformTag, PythonInterpreter, Target};
+#[cfg(feature = "zig")]
+use crate::PlatformTag;
+use crate::{BuildContext, PythonInterpreter, Target};
 use anyhow::{anyhow, bail, Context, Result};
 use fat_macho::FatWriter;
 use fs_err::{self as fs, File};
+use normpath::PathExt;
 use std::collections::HashMap;
 use std::env;
 use std::io::{BufReader, Read};
@@ -19,7 +22,14 @@ const PYO3_ABI3_NO_PYTHON_VERSION: (u64, u64, u64) = (0, 16, 4);
 /// crate types excluding `bin`, `cdylib` and `proc-macro`
 pub(crate) const LIB_CRATE_TYPES: [&str; 4] = ["lib", "dylib", "rlib", "staticlib"];
 
-pub(crate) type CompileTarget<'a> = (&'a cargo_metadata::Target, &'a BridgeModel);
+/// A cargo target to build
+#[derive(Debug, Clone)]
+pub struct CompileTarget {
+    /// The cargo target to build
+    pub target: cargo_metadata::Target,
+    /// The bridge model to use
+    pub bridge_model: BridgeModel,
+}
 
 /// A cargo build artifact
 #[derive(Debug, Clone)]
@@ -34,10 +44,10 @@ pub struct BuildArtifact {
 /// Builds the rust crate into a native module (i.e. an .so or .dll) for a
 /// specific python version. Returns a mapping from crate type (e.g. cdylib)
 /// to artifact location.
-pub fn compile<'a>(
-    context: &'a BuildContext,
+pub fn compile(
+    context: &BuildContext,
     python_interpreter: Option<&PythonInterpreter>,
-    targets: &[CompileTarget<'a>],
+    targets: &[CompileTarget],
 ) -> Result<Vec<HashMap<String, BuildArtifact>>> {
     if context.target.is_macos() && context.universal2 {
         compile_universal2(context, python_interpreter, targets)
@@ -47,10 +57,10 @@ pub fn compile<'a>(
 }
 
 /// Build an universal2 wheel for macos which contains both an x86 and an aarch64 binary
-fn compile_universal2<'a>(
-    context: &'a BuildContext,
+fn compile_universal2(
+    context: &BuildContext,
     python_interpreter: Option<&PythonInterpreter>,
-    targets: &[CompileTarget<'a>],
+    targets: &[CompileTarget],
 ) -> Result<Vec<HashMap<String, BuildArtifact>>> {
     let mut aarch64_context = context.clone();
     aarch64_context.target = Target::from_target_triple(Some("aarch64-apple-darwin".to_string()))?;
@@ -66,7 +76,7 @@ fn compile_universal2<'a>(
     let mut universal_artifacts = Vec::with_capacity(targets.len());
     for (bridge_model, (aarch64_artifact, x86_64_artifact)) in targets
         .iter()
-        .map(|(_, bridge_model)| bridge_model)
+        .map(|target| &target.bridge_model)
         .zip(aarch64_artifacts.iter().zip(&x86_64_artifacts))
     {
         let build_type = if bridge_model.is_bin() {
@@ -124,19 +134,14 @@ fn compile_universal2<'a>(
     Ok(universal_artifacts)
 }
 
-fn compile_targets<'a>(
-    context: &'a BuildContext,
+fn compile_targets(
+    context: &BuildContext,
     python_interpreter: Option<&PythonInterpreter>,
-    targets: &[CompileTarget<'a>],
+    targets: &[CompileTarget],
 ) -> Result<Vec<HashMap<String, BuildArtifact>>> {
     let mut artifacts = Vec::with_capacity(targets.len());
-    for (target, bridge_model) in targets {
-        artifacts.push(compile_target(
-            context,
-            python_interpreter,
-            bridge_model,
-            target,
-        )?);
+    for target in targets {
+        artifacts.push(compile_target(context, python_interpreter, target)?);
     }
     Ok(artifacts)
 }
@@ -144,13 +149,12 @@ fn compile_targets<'a>(
 fn compile_target(
     context: &BuildContext,
     python_interpreter: Option<&PythonInterpreter>,
-    bridge_model: &BridgeModel,
-    binding_target: &cargo_metadata::Target,
+    compile_target: &CompileTarget,
 ) -> Result<HashMap<String, BuildArtifact>> {
     let target = &context.target;
 
     let mut cargo_rustc: cargo_options::Rustc = context.cargo_options.clone().into();
-    cargo_rustc.message_format = vec!["json".to_string()];
+    cargo_rustc.message_format = vec!["json-render-diagnostics".to_string()];
 
     // --release and --profile are conflicting options
     if context.release && cargo_rustc.profile.is_none() {
@@ -158,7 +162,8 @@ fn compile_target(
     }
 
     // Add `--crate-type cdylib` if available
-    if binding_target
+    if compile_target
+        .target
         .kind
         .iter()
         .any(|k| LIB_CRATE_TYPES.contains(&k.as_str()))
@@ -171,12 +176,18 @@ fn compile_target(
         }
     }
 
-    let mut rust_flags = env::var_os("RUSTFLAGS");
+    let target_triple = target.target_triple();
+
+    let manifest_dir = context.manifest_path.parent().unwrap();
+    let mut rustflags = cargo_config2::Config::load_with_cwd(manifest_dir)?
+        .rustflags(target_triple)?
+        .unwrap_or_default();
 
     // We need to pass --bin / --lib
+    let bridge_model = &compile_target.bridge_model;
     match bridge_model {
         BridgeModel::Bin(..) => {
-            cargo_rustc.bin.push(binding_target.name.clone());
+            cargo_rustc.bin.push(compile_target.target.name.clone());
         }
         BridgeModel::Cffi
         | BridgeModel::UniFfi
@@ -186,11 +197,15 @@ fn compile_target(
             // https://github.com/rust-lang/rust/issues/59302#issue-422994250
             // We must only do this for libraries as it breaks binaries
             // For some reason this value is ignored when passed as rustc argument
-            if context.target.is_musl_target() {
+            if context.target.is_musl_target()
+                && !rustflags
+                    .flags
+                    .iter()
+                    .any(|f| f == "target-feature=-crt-static")
+            {
                 debug!("Setting `-C target-features=-crt-static` for musl dylib");
-                rust_flags
-                    .get_or_insert_with(Default::default)
-                    .push(" -C target-feature=-crt-static");
+                rustflags.push("-C");
+                rustflags.push("target-feature=-crt-static");
             }
         }
     }
@@ -222,10 +237,11 @@ fn compile_target(
             cargo_rustc.args.extend(mac_args);
         }
     } else if target.is_emscripten() {
-        let flags = rust_flags.get_or_insert_with(Default::default);
         // Allow user to override these default flags
-        if !flags.to_string_lossy().contains("link-native-libraries") {
-            flags.push(" -Z link-native-libraries=no");
+        if !rustflags.flags.iter().any(|f| f == "link-native-libraries") {
+            debug!("Setting `-Z link-native-libraries=no` for Emscripten");
+            rustflags.push("-Z");
+            rustflags.push("link-native-libraries=no");
         }
         let mut emscripten_args = Vec::new();
         // Allow user to override these default settings
@@ -258,36 +274,56 @@ fn compile_target(
             .extend(["-C".to_string(), "link-arg=-s".to_string()]);
     }
 
-    let target_triple = target.target_triple();
     let mut build_command = if target.is_msvc() && target.cross_compiling() {
-        let mut build = cargo_xwin::Rustc::from(cargo_rustc);
+        #[cfg(feature = "xwin")]
+        {
+            let mut build = cargo_xwin::Rustc::from(cargo_rustc);
 
-        build.target = vec![target_triple.to_string()];
-        build.build_command()?
-    } else {
-        let mut build = cargo_zigbuild::Rustc::from(cargo_rustc);
-        if !context.zig {
-            build.disable_zig_linker = true;
+            build.target = vec![target_triple.to_string()];
+            build.build_command()?
+        }
+        #[cfg(not(feature = "xwin"))]
+        {
             if target.user_specified {
-                build.target = vec![target_triple.to_string()];
+                cargo_rustc.target = vec![target_triple.to_string()];
             }
-        } else {
-            build.enable_zig_ar = true;
-            let zig_triple = if target.is_linux() && !target.is_musl_target() {
-                match context.platform_tag.iter().find(|tag| tag.is_manylinux()) {
-                    Some(PlatformTag::Manylinux { x, y }) => {
-                        format!("{target_triple}.{x}.{y}")
-                    }
-                    _ => target_triple.to_string(),
+            cargo_rustc.command()
+        }
+    } else {
+        #[cfg(feature = "zig")]
+        {
+            let mut build = cargo_zigbuild::Rustc::from(cargo_rustc);
+            if !context.zig {
+                build.disable_zig_linker = true;
+                if target.user_specified {
+                    build.target = vec![target_triple.to_string()];
                 }
             } else {
-                target_triple.to_string()
-            };
-            build.target = vec![zig_triple];
+                build.enable_zig_ar = true;
+                let zig_triple = if target.is_linux() && !target.is_musl_target() {
+                    match context.platform_tag.iter().find(|tag| tag.is_manylinux()) {
+                        Some(PlatformTag::Manylinux { x, y }) => {
+                            format!("{target_triple}.{x}.{y}")
+                        }
+                        _ => target_triple.to_string(),
+                    }
+                } else {
+                    target_triple.to_string()
+                };
+                build.target = vec![zig_triple];
+            }
+            build.build_command()?
+        }
+        #[cfg(not(feature = "zig"))]
+        {
+            if target.user_specified {
+                cargo_rustc.target = vec![target_triple.to_string()];
+            }
+            cargo_rustc.command()
         }
-        build.build_command()?
     };
 
+    #[cfg(feature = "zig")]
     if context.zig {
         // Pass zig command to downstream, eg. python3-dll-a
         if let Ok((zig_cmd, zig_args)) = cargo_zigbuild::Zig::find_zig() {
@@ -309,8 +345,8 @@ fn compile_target(
         // but forwarding stderr is still useful in case there some non-json error
         .stderr(Stdio::inherit());
 
-    if let Some(flags) = rust_flags {
-        build_command.env("RUSTFLAGS", flags);
+    if !rustflags.flags.is_empty() {
+        build_command.env("CARGO_ENCODED_RUSTFLAGS", rustflags.encode()?);
     }
 
     if let BridgeModel::BindingsAbi3(_, _) = bridge_model {
@@ -334,9 +370,12 @@ fn compile_target(
         if interpreter.runnable {
             if bridge_model.is_bindings("pyo3")
                 || bridge_model.is_bindings("pyo3-ffi")
-                || (matches!(bridge_model, BridgeModel::BindingsAbi3(_, _))
-                    && interpreter.interpreter_kind.is_pypy())
+                || matches!(bridge_model, BridgeModel::BindingsAbi3(_, _))
             {
+                debug!(
+                    "Setting PYO3_PYTHON to {}",
+                    interpreter.executable.display()
+                );
                 build_command
                     .env("PYO3_PYTHON", &interpreter.executable)
                     .env(
@@ -366,7 +405,8 @@ fn compile_target(
                     config_file.display()
                 )
             })?;
-            build_command.env("PYO3_CONFIG_FILE", config_file);
+            let abs_config_file = config_file.normalize()?.into_path_buf();
+            build_command.env("PYO3_CONFIG_FILE", abs_config_file);
         }
     }
 
@@ -376,13 +416,28 @@ fn compile_target(
 
     // Set default macOS deployment target version
     if target.is_macos() && env::var_os("MACOSX_DEPLOYMENT_TARGET").is_none() {
-        use crate::target::rustc_macosx_target_version;
+        use crate::build_context::rustc_macosx_target_version;
 
-        let (major, minor) = rustc_macosx_target_version(target_triple);
-        build_command.env("MACOSX_DEPLOYMENT_TARGET", format!("{major}.{minor}"));
-        eprintln!(
-            "💻 Using `MACOSX_DEPLOYMENT_TARGET={major}.{minor}` for {target_triple} by default"
-        );
+        let target_config = context
+            .pyproject_toml
+            .as_ref()
+            .and_then(|x| x.target_config(target_triple));
+        let deployment_target = if let Some(deployment_target) = target_config
+            .as_ref()
+            .and_then(|config| config.macos_deployment_target.as_ref())
+        {
+            eprintln!(
+                "💻 Using `MACOSX_DEPLOYMENT_TARGET={deployment_target}` for {target_triple} by configuration"
+            );
+            deployment_target.clone()
+        } else {
+            let (major, minor) = rustc_macosx_target_version(target_triple);
+            eprintln!(
+                "💻 Using `MACOSX_DEPLOYMENT_TARGET={major}.{minor}` for {target_triple} by default"
+            );
+            format!("{major}.{minor}")
+        };
+        build_command.env("MACOSX_DEPLOYMENT_TARGET", deployment_target);
     }
 
     debug!("Running {:?}", build_command);
