@@ -4,16 +4,20 @@
 //! Run with --help for usage information
 
 use anyhow::{bail, Context, Result};
+#[cfg(feature = "zig")]
 use cargo_zigbuild::Zig;
-use clap::{CommandFactory, Parser, Subcommand};
+#[cfg(feature = "cli-completion")]
+use clap::CommandFactory;
+use clap::{Parser, Subcommand};
+#[cfg(feature = "scaffolding")]
+use maturin::{ci::GenerateCI, init_project, new_project, GenerateProjectOptions};
 use maturin::{
-    ci::GenerateCI, develop, init_project, new_project, write_dist_info, BridgeModel, BuildOptions,
-    CargoOptions, GenerateProjectOptions, PathWriter, PlatformTag, PythonInterpreter, Target,
+    develop, write_dist_info, BridgeModel, BuildOptions, CargoOptions, PathWriter, PlatformTag,
+    PythonInterpreter, Target,
 };
 #[cfg(feature = "upload")]
 use maturin::{upload_ui, PublishOpt};
 use std::env;
-use std::io;
 use std::path::PathBuf;
 use tracing::debug;
 
@@ -115,6 +119,7 @@ enum Opt {
         out: Option<PathBuf>,
     },
     /// Create a new cargo project in an existing directory
+    #[cfg(feature = "scaffolding")]
     #[command(name = "init")]
     InitProject {
         /// Project path
@@ -123,6 +128,7 @@ enum Opt {
         options: GenerateProjectOptions,
     },
     /// Create a new cargo project
+    #[cfg(feature = "scaffolding")]
     #[command(name = "new")]
     NewProject {
         /// Project path
@@ -130,6 +136,7 @@ enum Opt {
         #[command(flatten)]
         options: GenerateProjectOptions,
     },
+    #[cfg(feature = "scaffolding")]
     #[command(name = "generate-ci")]
     GenerateCI(GenerateCI),
     /// Upload python packages to pypi
@@ -151,12 +158,14 @@ enum Opt {
     #[command(subcommand)]
     Pep517(Pep517Command),
     /// Generate shell completions
+    #[cfg(feature = "cli-completion")]
     #[command(name = "completions", hide = true)]
     Completions {
         #[arg(value_name = "SHELL")]
         shell: clap_complete_command::Shell,
     },
     /// Zig linker wrapper
+    #[cfg(feature = "zig")]
     #[command(subcommand, hide = true)]
     Zig(Zig),
 }
@@ -265,25 +274,16 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
             let context = build_options.into_build_context(true, strip, false)?;
 
             // Since afaik all other PEP 517 backends also return linux tagged wheels, we do so too
-            let tags = match context.bridge {
+            let tags = match context.bridge() {
                 BridgeModel::Bindings(..) | BridgeModel::Bin(Some(..)) => {
-                    vec![context.interpreter[0].get_tag(
-                        &context.target,
-                        &[PlatformTag::Linux],
-                        context.universal2,
-                    )?]
+                    vec![context.interpreter[0].get_tag(&context, &[PlatformTag::Linux])?]
                 }
                 BridgeModel::BindingsAbi3(major, minor) => {
-                    let platform = context
-                        .target
-                        .get_platform_tag(&[PlatformTag::Linux], context.universal2)?;
+                    let platform = context.get_platform_tag(&[PlatformTag::Linux])?;
                     vec![format!("cp{major}{minor}-abi3-{platform}")]
                 }
                 BridgeModel::Bin(None) | BridgeModel::Cffi | BridgeModel::UniFfi => {
-                    context
-                        .target
-                        .get_universal_tags(&[PlatformTag::Linux], context.universal2)?
-                        .1
+                    context.get_universal_tags(&[PlatformTag::Linux])?.1
                 }
             };
 
@@ -328,20 +328,26 @@ fn run() -> Result<()> {
     #[cfg(feature = "log")]
     tracing_subscriber::fmt::init();
 
-    // Allow symlink `maturin` to `ar` to invoke `zig ar`
-    // See https://github.com/messense/cargo-zigbuild/issues/52
-    let mut args = env::args();
-    let program_path = PathBuf::from(args.next().expect("no program path"));
-    let program_name = program_path.file_stem().expect("no program name");
-    if program_name.eq_ignore_ascii_case("ar") {
-        let zig = Zig::Ar {
-            args: args.collect(),
-        };
-        zig.execute()?;
-        return Ok(());
+    #[cfg(feature = "zig")]
+    {
+        // Allow symlink `maturin` to `ar` to invoke `zig ar`
+        // See https://github.com/messense/cargo-zigbuild/issues/52
+        let mut args = env::args();
+        let program_path = PathBuf::from(args.next().expect("no program path"));
+        let program_name = program_path.file_stem().expect("no program name");
+        if program_name.eq_ignore_ascii_case("ar") {
+            let zig = Zig::Ar {
+                args: args.collect(),
+            };
+            zig.execute()?;
+            return Ok(());
+        }
     }
 
+    #[cfg(not(feature = "wild"))]
     let opt = Opt::parse();
+    #[cfg(feature = "wild")]
+    let opt = Opt::parse_from(wild::args_os());
 
     match opt {
         Opt::Build {
@@ -424,8 +430,11 @@ fn run() -> Result<()> {
                 .context("Failed to build source distribution, pyproject.toml not found")?;
         }
         Opt::Pep517(subcommand) => pep517(subcommand)?,
+        #[cfg(feature = "scaffolding")]
         Opt::InitProject { path, options } => init_project(path, options)?,
+        #[cfg(feature = "scaffolding")]
         Opt::NewProject { path, options } => new_project(path, options)?,
+        #[cfg(feature = "scaffolding")]
         Opt::GenerateCI(generate_ci) => generate_ci.execute()?,
         #[cfg(feature = "upload")]
         Opt::Upload { publish, files } => {
@@ -436,9 +445,11 @@ fn run() -> Result<()> {
 
             upload_ui(&files, &publish)?
         }
+        #[cfg(feature = "cli-completion")]
         Opt::Completions { shell } => {
-            shell.generate(&mut Opt::command(), &mut io::stdout());
+            shell.generate(&mut Opt::command(), &mut std::io::stdout());
         }
+        #[cfg(feature = "zig")]
         Opt::Zig(subcommand) => {
             subcommand
                 .execute()
@@ -477,7 +488,7 @@ fn main() {
 
     if let Err(e) = run() {
         eprintln!("💥 maturin failed");
-        for cause in e.chain().collect::<Vec<_>>().iter() {
+        for cause in e.chain() {
             eprintln!("  Caused by: {cause}");
         }
         std::process::exit(1);
