@@ -108,14 +108,12 @@ impl ProjectResolver {
 
         let cargo_metadata = Self::resolve_cargo_metadata(&manifest_file, &cargo_options)?;
 
-        let mut metadata21 =
-            Metadata21::from_cargo_toml(&cargo_toml, manifest_dir, &cargo_metadata)
-                .context("Failed to parse Cargo.toml into python metadata")?;
+        let mut metadata21 = Metadata21::from_cargo_toml(manifest_dir, &cargo_metadata)
+            .context("Failed to parse Cargo.toml into python metadata")?;
         if let Some(pyproject) = pyproject {
             let pyproject_dir = pyproject_file.parent().unwrap();
             metadata21.merge_pyproject_toml(pyproject_dir, pyproject)?;
         }
-        let extra_metadata = cargo_toml.remaining_core_metadata();
 
         let crate_name = &cargo_toml.package.name;
 
@@ -130,13 +128,8 @@ impl ProjectResolver {
 
         let extension_name = pyproject
             .and_then(|x| x.module_name())
-            .or_else(|| {
-                if extra_metadata.name.is_some() {
-                    eprintln!("⚠️ Warning: specify [package.metadata.maturin] name in Cargo.toml is deprecated, use module-name in [tool.maturin] section in pyproject.toml instead");
-                }
-                extra_metadata.name.as_deref()
-            })
             .unwrap_or(&module_name);
+
         let project_root = if pyproject_file.is_file() {
             pyproject_file.parent().unwrap_or(manifest_dir)
         } else {
@@ -148,57 +141,40 @@ impl ProjectResolver {
             .to_vec();
         let py_root = match pyproject.and_then(|x| x.python_source()) {
             Some(py_src) => project_root.join(py_src),
-            None => match extra_metadata.python_source.as_ref() {
-                Some(py_src) => {
-                    eprintln!("⚠️ Warning: specify python-source in Cargo.toml is deprecated, use python-source in [tool.maturin] section in pyproject.toml instead");
-                    manifest_dir.join(py_src)
-                }
-                None => match pyproject.and_then(|x| x.project_name()) {
-                    Some(project_name) => {
-                        // Detect src layout
-                        let rust_cargo_toml_found =
-                            project_root.join("rust").join("Cargo.toml").is_file();
-                        let import_name = project_name.replace('-', "_");
-                        let mut package_init = HashSet::new();
-                        package_init.insert(
-                            project_root
-                                .join("src")
-                                .join(import_name)
-                                .join("__init__.py"),
-                        );
-                        for package in &python_packages {
-                            package_init
-                                .insert(project_root.join("src").join(package).join("__init__.py"));
-                        }
-                        let python_src_found = package_init.iter().any(|x| x.is_file());
-                        if rust_cargo_toml_found && python_src_found {
-                            project_root.join("src")
-                        } else {
-                            project_root.to_path_buf()
-                        }
+            None => match pyproject.and_then(|x| x.project_name()) {
+                Some(project_name) => {
+                    // Detect src layout
+                    let rust_cargo_toml_found =
+                        project_root.join("rust").join("Cargo.toml").is_file();
+                    let import_name = project_name.replace('-', "_");
+                    let mut package_init = HashSet::new();
+                    package_init.insert(
+                        project_root
+                            .join("src")
+                            .join(import_name)
+                            .join("__init__.py"),
+                    );
+                    for package in &python_packages {
+                        package_init
+                            .insert(project_root.join("src").join(package).join("__init__.py"));
                     }
-                    None => project_root.to_path_buf(),
-                },
+                    let python_src_found = package_init.iter().any(|x| x.is_file());
+                    if rust_cargo_toml_found && python_src_found {
+                        project_root.join("src")
+                    } else {
+                        project_root.to_path_buf()
+                    }
+                }
+                None => project_root.to_path_buf(),
             },
         };
-        let data = match pyproject.and_then(|x| x.data()) {
-            Some(data) => {
-                if data.is_absolute() {
-                    Some(data.to_path_buf())
-                } else {
-                    Some(project_root.join(data))
-                }
+        let data = pyproject.and_then(|x| x.data()).map(|data| {
+            if data.is_absolute() {
+                data.to_path_buf()
+            } else {
+                project_root.join(data)
             }
-            None => extra_metadata.data.as_ref().map(|data| {
-                eprintln!("⚠️ Warning: specify `data` in Cargo.toml is deprecated, use `data` in [tool.maturin] section in pyproject.toml instead");
-                let data = Path::new(data);
-                if data.is_absolute() {
-                    data.to_path_buf()
-                } else {
-                    manifest_dir.join(data)
-                }
-            }),
-        };
+        });
         let project_layout =
             ProjectLayout::determine(project_root, extension_name, py_root, python_packages, data)?;
         Ok(Self {
