@@ -1,4 +1,5 @@
 use crate::module_writer::{add_data, ModuleWriter};
+use crate::pyproject_toml::SdistGenerator;
 use crate::{pyproject_toml::Format, BuildContext, PyProjectToml, SDistWriter};
 use anyhow::{bail, Context, Result};
 use cargo_metadata::{Metadata, MetadataCommand};
@@ -55,10 +56,132 @@ fn rewrite_cargo_toml(
         "Can't read Cargo.toml at {}",
         manifest_path.display(),
     ))?;
-    let mut data = text.parse::<toml_edit::Document>().context(format!(
+    let mut document = text.parse::<toml_edit::Document>().context(format!(
         "Failed to parse Cargo.toml at {}",
         manifest_path.display()
     ))?;
+
+    let workspace = workspace_manifest.get("workspace");
+    let workspace_deps = workspace
+        .and_then(|x| x.get("dependencies"))
+        .and_then(|x| x.as_table_like());
+
+    let mut rewritten = rewrite_dependencies_path(
+        document.as_table_mut(),
+        workspace_deps,
+        manifest_path,
+        known_path_deps,
+        &local_deps_folder,
+        root_crate,
+    )?;
+    if let Some(target_specs) = document
+        .get_mut("target")
+        .and_then(|x| x.as_table_like_mut())
+    {
+        for target_spec in target_specs
+            .iter_mut()
+            .filter_map(|x| x.1.as_table_like_mut())
+        {
+            rewritten |= rewrite_dependencies_path(
+                target_spec,
+                workspace_deps,
+                manifest_path,
+                known_path_deps,
+                &local_deps_folder,
+                root_crate,
+            )?;
+        }
+    }
+
+    // Update workspace inherited metadata
+    if let Some(package) = document.get_mut("package").and_then(|x| x.as_table_mut()) {
+        let workspace_package = workspace
+            .and_then(|x| x.get("package"))
+            .and_then(|x| x.as_table_like());
+        for key in WORKSPACE_INHERITABLE_FIELDS.iter().copied() {
+            let workspace_inherited = package
+                .get(key)
+                .and_then(|x| x.get("workspace"))
+                .and_then(|x| x.as_bool())
+                .unwrap_or_default();
+            if workspace_inherited {
+                if let Some(workspace_value) = workspace_package.and_then(|ws| ws.get(key)) {
+                    package[key] = workspace_value.clone();
+                    rewritten = true;
+                }
+            }
+        }
+
+        // Update resolver if workspace set one
+        if let Some(resolver) = workspace.and_then(|x| x.get("resolver")) {
+            package["resolver"] = resolver.clone();
+            rewritten = true;
+        }
+    }
+
+    if root_crate {
+        // Update workspace members
+        if let Some(workspace) = document.get_mut("workspace").and_then(|x| x.as_table_mut()) {
+            if let Some(members) = workspace.get_mut("members").and_then(|x| x.as_array_mut()) {
+                if known_path_deps.is_empty() {
+                    // Remove workspace members when there isn't any path dep
+                    workspace.remove("members");
+                    if workspace.is_empty() {
+                        // Remove workspace all together if it's empty
+                        document.remove("workspace");
+                    }
+                    rewritten = true;
+                } else {
+                    let mut new_members = toml_edit::Array::new();
+                    for member in members.iter() {
+                        if let toml_edit::Value::String(ref s) = member {
+                            let path = Path::new(s.value());
+                            if let Some(name) = path.file_name().and_then(|x| x.to_str()) {
+                                if known_path_deps.contains_key(name) {
+                                    new_members.push(format!("{LOCAL_DEPENDENCIES_FOLDER}/{name}"));
+                                }
+                            }
+                        }
+                    }
+                    if !new_members.is_empty() {
+                        workspace["members"] = toml_edit::value(new_members);
+                    } else {
+                        workspace.remove("members");
+                    }
+                    rewritten = true;
+                }
+            }
+        }
+    } else {
+        // Update package.workspace
+        // https://rust-lang.github.io/rfcs/1525-cargo-workspace.html#implicit-relations
+        // https://doc.rust-lang.org/cargo/reference/manifest.html#the-workspace-field
+        if let Some(package) = document.get_mut("package").and_then(|x| x.as_table_mut()) {
+            if let Some(workspace) = package.get("workspace").and_then(|x| x.as_str()) {
+                // This is enough to fix https://github.com/PyO3/maturin/issues/838
+                // Other cases can be fixed on demand
+                if workspace == ".." || workspace == "../" {
+                    package.remove("workspace");
+                    rewritten = true;
+                }
+            }
+        }
+    }
+    if rewritten {
+        Ok(document.to_string())
+    } else {
+        Ok(text)
+    }
+}
+
+fn rewrite_dependencies_path(
+    table: &mut dyn toml_edit::TableLike,
+    workspace_deps: Option<&dyn toml_edit::TableLike>,
+    manifest_path: &Path,
+    known_path_deps: &HashMap<String, PathBuf>,
+    local_deps_folder: &str,
+    root_crate: bool,
+) -> Result<bool> {
     let mut rewritten = false;
     //  ˇˇˇˇˇˇˇˇˇˇˇˇ dep_category
     // [dependencies]
@@ -66,19 +189,15 @@ fn rewrite_cargo_toml(
     //                          ^^^^^^^^^^^^^^^^^^ table[&dep_name]["path"]
     // ^^^^^^^^^^^^^ dep_name
     for dep_category in ["dependencies", "dev-dependencies", "build-dependencies"] {
-        if let Some(table) = data.get_mut(dep_category).and_then(|x| x.as_table_mut()) {
+        if let Some(table) = table.get_mut(dep_category).and_then(|x| x.as_table_mut()) {
             if dep_category == "dev-dependencies" && !known_path_deps.is_empty() {
                 // Remove dev-dependencies since building from sdist doesn't need them,
                 // Keep it when there are no path dependencies to support building from
                 // sdist with `--locked`/`--frozen`.
-                data.remove(dep_category);
+                table.remove(dep_category);
                 rewritten = true;
                 continue;
             }
-            let workspace_deps = workspace_manifest
-                .get("workspace")
-                .and_then(|x| x.get("dependencies"))
-                .and_then(|x| x.as_table_like());
             let dep_names: Vec<_> = table.iter().map(|(key, _)| key.to_string()).collect();
             for dep_name in dep_names {
                 let workspace_inherit = table
@@ -134,11 +253,11 @@ fn rewrite_cargo_toml(
                                     .as_array_mut()
                                     .with_context(|| {
                                         format!(
-                                            "In {}, {} {} has a features value that is not an array",
-                                            manifest_path.display(),
-                                            dep_category,
-                                            dep_name
-                                        )
+                                        "In {}, {} {} has a features value that is not an array",
+                                        manifest_path.display(),
+                                        dep_category,
+                                        dep_name
+                                    )
                                     })?;
                                 existing_features.extend(features);
                             }
@@ -146,12 +265,12 @@ fn rewrite_cargo_toml(
                             rewritten = true;
                         } else {
                             bail!(
-                                "In {}, {} {} is marked as `workspace = true`, but it is found neither in \
+                            "In {}, {} {} is marked as `workspace = true`, but it is found neither in \
                                 the workspace manifest nor in the known path dependencies",
-                                manifest_path.display(),
-                                dep_category,
-                                dep_name
-                            )
+                            manifest_path.display(),
+                            dep_category,
+                            dep_name
+                        )
                         }
                         continue;
                     }
@@ -174,81 +293,7 @@ fn rewrite_cargo_toml(
             }
         }
     }
-
-    // Update workspace inherited metadata
-    if let Some(package) = data.get_mut("package").and_then(|x| x.as_table_mut()) {
-        let workspace_package = workspace_manifest
-            .get("workspace")
-            .and_then(|x| x.get("package"))
-            .and_then(|x| x.as_table_like());
-        for key in WORKSPACE_INHERITABLE_FIELDS.iter().copied() {
-            let workspace_inherited = package
-                .get(key)
-                .and_then(|x| x.get("workspace"))
-                .and_then(|x| x.as_bool())
-                .unwrap_or_default();
-            if workspace_inherited {
-                if let Some(workspace_value) = workspace_package.and_then(|ws| ws.get(key)) {
-                    package[key] = workspace_value.clone();
-                    rewritten = true;
-                }
-            }
-        }
-    }
-
-    if root_crate {
-        // Update workspace members
-        if let Some(workspace) = data.get_mut("workspace").and_then(|x| x.as_table_mut()) {
-            if let Some(members) = workspace.get_mut("members").and_then(|x| x.as_array_mut()) {
-                if known_path_deps.is_empty() {
-                    // Remove workspace members when there isn't any path dep
-                    workspace.remove("members");
-                    if workspace.is_empty() {
-                        // Remove workspace all together if it's empty
-                        data.remove("workspace");
-                    }
-                    rewritten = true;
-                } else {
-                    let mut new_members = toml_edit::Array::new();
-                    for member in members.iter() {
-                        if let toml_edit::Value::String(ref s) = member {
-                            let path = Path::new(s.value());
-                            if let Some(name) = path.file_name().and_then(|x| x.to_str()) {
-                                if known_path_deps.contains_key(name) {
-                                    new_members.push(format!("{LOCAL_DEPENDENCIES_FOLDER}/{name}"));
-                                }
-                            }
-                        }
-                    }
-                    if !new_members.is_empty() {
-                        workspace["members"] = toml_edit::value(new_members);
-                    } else {
-                        workspace.remove("members");
-                    }
-                    rewritten = true;
-                }
-            }
-        }
-    } else {
-        // Update package.workspace
-        // https://rust-lang.github.io/rfcs/1525-cargo-workspace.html#implicit-relations
-        // https://doc.rust-lang.org/cargo/reference/manifest.html#the-workspace-field
-        if let Some(package) = data.get_mut("package").and_then(|x| x.as_table_mut()) {
-            if let Some(workspace) = package.get("workspace").and_then(|x| x.as_str()) {
-                // This is enough to fix https://github.com/PyO3/maturin/issues/838
-                // Other cases can be fixed on demand
-                if workspace == ".." || workspace == "../" {
-                    package.remove("workspace");
-                    rewritten = true;
-                }
-            }
-        }
-    }
-    if rewritten {
-        Ok(data.to_string())
-    } else {
-        Ok(text)
-    }
+    Ok(rewritten)
 }
 
 /// Make sure that the dep entry is an inline table
@@ -395,7 +440,13 @@ fn add_crate_to_source_distribution(
             .any(|(target, _)| target == Path::new("pyproject.toml"))
     {
         // Add pyproject.toml to the source distribution
-        if cargo_toml_in_subdir {
+        // `pyproject.toml` may not be included in `cargo package --list`
+        // (e.g. it's not specified in `include` or is specified in `exclude` by mistake)
+        // we check if it's the same pyproject.toml file in Cargo.toml directory
+        let pyproject_toml_in_manifest_dir =
+            same_file::is_same_file(pyproject_toml_path, abs_manifest_dir.join("pyproject.toml"))
+                .unwrap_or(false);
+        if cargo_toml_in_subdir || pyproject_toml_in_manifest_dir {
             // if Cargo.toml is in subdirectory of pyproject.toml directory
             target_source.push((
                 PathBuf::from("pyproject.toml"),
@@ -514,29 +565,53 @@ fn find_path_deps(cargo_metadata: &Metadata) -> Result<HashMap<String, PathBuf>>
     Ok(path_deps)
 }
 
-/// Creates a source distribution, packing the root crate and all local dependencies
+/// Copies the files of git to a source distribution
 ///
-/// The source distribution format is specified in
-/// [PEP 517 under "build_sdist"](https://www.python.org/dev/peps/pep-0517/#build-sdist)
-/// and in
-/// https://packaging.python.org/specifications/source-distribution-format/#source-distribution-file-format
-pub fn source_distribution(
+/// Runs `git ls-files -z` to obtain a list of files to package.
+fn add_git_tracked_files_to_sdist(
+    pyproject_toml_path: &Path,
+    writer: &mut SDistWriter,
+    prefix: impl AsRef<Path>,
+) -> Result<()> {
+    let pyproject_dir = pyproject_toml_path.parent().unwrap();
+    let output = Command::new("git")
+        .args(["ls-files", "-z"])
+        .current_dir(pyproject_dir)
+        .output()
+        .context("Failed to run `git ls-files -z`")?;
+    if !output.status.success() {
+        bail!(
+            "Failed to query file list from git: {}\n--- Project Path: {}\n--- Stdout:\n{}\n--- Stderr:\n{}",
+            output.status,
+            pyproject_dir.display(),
+            String::from_utf8_lossy(&output.stdout),
+            String::from_utf8_lossy(&output.stderr),
+        );
+    }
+
+    let prefix = prefix.as_ref();
+    writer.add_directory(prefix)?;
+
+    let file_paths = str::from_utf8(&output.stdout)
+        .context("git printed invalid utf-8 ಠ_ಠ")?
+        .split('\0')
+        .filter(|s| !s.is_empty())
+        .map(Path::new);
+    for source in file_paths {
+        writer.add_file(prefix.join(source), pyproject_dir.join(source))?;
+    }
+    Ok(())
+}
+
+/// Copies the files of a crate to a source distribution, recursively adding path dependencies
+/// and rewriting path entries in Cargo.toml
+fn add_cargo_package_files_to_sdist(
     build_context: &BuildContext,
-    pyproject: &PyProjectToml,
-    excludes: Option<Override>,
-) -> Result<PathBuf> {
-    let metadata21 = &build_context.metadata21;
+    pyproject_toml_path: &Path,
+    writer: &mut SDistWriter,
+    root_dir: &Path,
+) -> Result<()> {
     let manifest_path = &build_context.manifest_path;
-    let pyproject_toml_path = build_context
-        .pyproject_toml_path
-        .normalize()
-        .with_context(|| {
-            format!(
-                "failed to normalize path `{}`",
-                build_context.pyproject_toml_path.display()
-            )
-        })?
-        .into_path_buf();
     let workspace_manifest_path = build_context
         .cargo_metadata
         .workspace_root
@@ -546,13 +621,6 @@ pub fn source_distribution(
 
     let known_path_deps = find_path_deps(&build_context.cargo_metadata)?;
 
-    let mut writer = SDistWriter::new(&build_context.out, metadata21, excludes)?;
-    let root_dir = PathBuf::from(format!(
-        "{}-{}",
-        &metadata21.get_distribution_escaped(),
-        &metadata21.get_version_escaped()
-    ));
-
     // Add local path dependencies
     let mut path_dep_workspace_manifests = HashMap::new();
     for (name, path_dep) in known_path_deps.iter() {
@@ -585,8 +653,8 @@ pub fn source_distribution(
                 &path_dep_workspace_manifests[&path_dep_metadata.workspace_root]
             };
         add_crate_to_source_distribution(
-            &mut writer,
-            &pyproject_toml_path,
+            writer,
+            pyproject_toml_path,
             path_dep,
             path_dep_workspace_manifest,
             &root_dir.join(LOCAL_DEPENDENCIES_FOLDER).join(name),
@@ -602,11 +670,11 @@ pub fn source_distribution(
 
     // Add the main crate
     add_crate_to_source_distribution(
-        &mut writer,
-        &pyproject_toml_path,
+        writer,
+        pyproject_toml_path,
         manifest_path,
         &workspace_manifest,
-        &root_dir,
+        root_dir,
         &known_path_deps,
         true,
     )?;
@@ -683,12 +751,67 @@ pub fn source_distribution(
         }
     }
 
+    Ok(())
+}
+
+/// Creates a source distribution, packing the root crate and all local dependencies
+///
+/// The source distribution format is specified in
+/// [PEP 517 under "build_sdist"](https://www.python.org/dev/peps/pep-0517/#build-sdist)
+/// and in
+/// https://packaging.python.org/specifications/source-distribution-format/#source-distribution-file-format
+pub fn source_distribution(
+    build_context: &BuildContext,
+    pyproject: &PyProjectToml,
+    excludes: Option<Override>,
+) -> Result<PathBuf> {
+    let pyproject_toml_path = build_context
+        .pyproject_toml_path
+        .normalize()
+        .with_context(|| {
+            format!(
+                "failed to normalize path `{}`",
+                build_context.pyproject_toml_path.display()
+            )
+        })?
+        .into_path_buf();
+    let metadata21 = &build_context.metadata21;
+    let mut writer = SDistWriter::new(&build_context.out, metadata21, excludes)?;
+    let root_dir = PathBuf::from(format!(
+        "{}-{}",
+        &metadata21.get_distribution_escaped(),
+        &metadata21.get_version_escaped()
+    ));
+
+    match pyproject.sdist_generator() {
+        SdistGenerator::Cargo => add_cargo_package_files_to_sdist(
+            build_context,
+            &pyproject_toml_path,
+            &mut writer,
+            &root_dir,
+        )?,
+        SdistGenerator::Git => {
+            add_git_tracked_files_to_sdist(&pyproject_toml_path, &mut writer, &root_dir)?
+        }
+    }
+
+    let pyproject_toml_path = build_context
+        .pyproject_toml_path
+        .normalize()
+        .with_context(|| {
+            format!(
+                "failed to normalize path `{}`",
+                build_context.pyproject_toml_path.display()
+            )
+        })?
+        .into_path_buf();
+    let pyproject_dir = pyproject_toml_path.parent().unwrap();
     // Add readme, license
     if let Some(project) = pyproject.project.as_ref() {
         if let Some(pyproject_toml::ReadMe::RelativePath(readme)) = project.readme.as_ref() {
             writer.add_file(root_dir.join(readme), pyproject_dir.join(readme))?;
         }
-        if let Some(pyproject_toml::License {
+        if let Some(pyproject_toml::License::Table {
             file: Some(license),
             text: None,
         }) = project.license.as_ref()
@@ -713,16 +836,6 @@ pub fn source_distribution(
         Ok(())
     };
 
-    #[allow(deprecated)]
-    if let Some(include_targets) = pyproject.sdist_include() {
-        eprintln!(
-            "⚠️  Warning: `[tool.maturin.sdist-include]` is deprecated, please use `[tool.maturin.include]`"
-        );
-        for pattern in include_targets {
-            include(pattern.as_str())?;
-        }
-    }
-
     if let Some(glob_patterns) = pyproject.include() {
         for pattern in glob_patterns
             .iter()
